<?php

class Model_Helper_TeacherBuilder extends Model_Helper_Builder
{
    public function create() {
        $this->data += [
            'SecondName'     => '',
            'JobPositionID'  => Model_Teacher::JOB_TEACHER,
            'DepID' => null,
        ];

        $required = [
            'LastName', 'SecondName', 'FirstName',
            'JobPositionID', 'DepID',
        ];

        if ( isset($this->data['ExternalID']) ) {
            $required += ['ExternalID'];
        } else {
            $this->data += [
                'ActivationCode' => Model_Account::generateActivationCode(),
            ];
            $required += ['ActivationCode'];
        }

        if (array_diff($required, array_keys($this->data)))
            throw new InvalidArgumentException('Not enough arguments');

        return new Model_Teacher($this->data, false);
    }

    function & lastName($string) {
        $string = trim($string);  # todo: remove empty symbols
        if (!strlen($string))
            throw new InvalidArgumentException('Last name can\'t be empty');
        $this->data['LastName'] = UTF8::clear($string);
        return $this;
    }

    function & firstName($string) {
        $string = trim($string);
        if (!strlen($string))
            throw new InvalidArgumentException('First name can\'t be empty');
        $this->data['FirstName'] = UTF8::clear($string);
        return $this;
    }

    function & secondName($string) {
        $this->data['SecondName'] = UTF8::clear($string);
        return $this;
    }

    function & inila($string) {
        $this->data['INILA'] = UTF8::clear($string);
        return $this;
    }

    function & jobPosition($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('Job position id is incorrect');
        $this->data['JobPositionID'] = (int) $id;
        return $this;
    }

    function & faculty($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('Faculty id is incorrect');
        $this->data['FacultyID'] = (int) $id;
        return $this;
    }

    function & department($id) {
        if (!is_numeric($id) || $id <= 0)
            throw new InvalidArgumentException('Department id is incorrect');
        $this->data['DepID'] = (int) $id;
        return $this;
    }

    function & externalID($extID) {
        $extID = trim($extID);  # todo: remove empty symbols
        if (!strlen($extID))
            throw new InvalidArgumentException('External ID can\'t be empty');
        $this->data['ExternalID'] = UTF8::clear($extID);
        return $this;
    }
}
