<?php defined('SYSPATH') || die('No direct script access.');

/**
 * Class Model_Student
 *
 * @property-read $ID             int
 * @property-read $LastName       string
 * @property-read $FirstName      string
 * @property      $SecondName     string
 * @property-read $AccountID      int
 * @property-read $SemesterID     int
 * @property-read $GroupID        int
 * @property-read $GroupNum       int
 * @property      $GroupName      string
 * @property-read $GradeID        int
 * @property      $GradeNum       int
 * @property      $Degree         string
 * @property      $SpecID         int
 * @property      $SpecName       string
 * @property      $SpecAbbr       string
 * @property      $SpecCode       string
 * @property-read $FacultyID      int
 * @property      $FacultyName    string
 * @property      $FacultyAbbr    string
 * @property-read $ActivationCode string
 * @property-read $RecordBooks array
 */
class Model_Student extends Model_Container
{
    protected function getRawData($id) {
        $this->data[self::$ID_FIELD] = $id;
        $sql = 'CALL `Student_GetInfo`(:id)';
        $info = DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)->execute();

        if ($info->count() == 0)
            throw new InvalidArgumentException('Student not found');

        $recordBooks = $this->getRecordBooks();
        $recordBooksMap = [];
        foreach ($recordBooks as $recordBook)
            $recordBooksMap[$recordBook['ID']] = $recordBook;

        return array_merge($info[0], array('RecordBooks' => $recordBooksMap));
    }

    public function getRecordBooks() {
        $sql = 'CALL `Student_GetRecordBooks`(:id)';
        return DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)->execute()->as_array();
    }

    public static function make() {
        return new Model_Helper_StudentBuilder();
    }

    protected function create() {
        $sql = 'SELECT `CreateOrAddStudent`(ExternalID, LastName, FirstName, SecondName) AS `ID`';
        $this->data[self::$ID_FIELD] = DB::query(Database::SELECT, $sql)
            ->parameters($this->data)
            ->execute()->get('ID');
    }

    public function update() {
        $updatedData = array_merge(['ExternalID' => null], $this->getRawData($this->ID), $this->data);
        $sql = 'SELECT `Student_ChangeInfo`(ID, ExternalID, LastName, FirstName, SecondName) AS `Success`';
        $success = DB::query(Database::SELECT, $sql)
            ->parameters($updatedData)
            ->execute()->get('Success');
        return $success == 0;
    }

    public static function createStudentEx($lastName, $firstName, $secondName, $gradeNum, $groupNum, $degree, $specialization, $facultyID, $semesterID = null) {
        $code = Model_Account::generateActivationCode();

        if (!$semesterID)
            $semesterID = Model_System::$SemesterID;

        $sql = 'SELECT `CreateStudentEx`(:last, :first, :second, :grade, :group, :degree, :spec, :faculty, :code, :semester) AS `UserID`;';
        $response = DB::query(Database::SELECT, $sql)
            ->parameters([
                ':last'     => trim($lastName),
                ':first'    => trim($firstName),
                ':second'   => trim($secondName),
                ':grade'    => $gradeNum,
                ':group'    => $groupNum,
                ':degree'   => $degree,
                ':spec'     => $specialization,
                ':faculty'  => $facultyID,
                ':code'     => $code,
                ':semester' => $semesterID
            ])->execute()->get('UserID');

        return $response == -1 ? -1 : $code;
    }

    /**
     * @param string $extid
     * @return integer
     */
    public static function withExternalID($extid) {
        $sql = 'SELECT `Student_GetIDFromExternalID`(:ExtID) AS `ID`';
        $res = DB::query(Database::SELECT, $sql)
            ->param(':ExtID', $extid)
            ->execute()->get('ID');
        return (int)$res;
    }

    /** @return Model_Discipline[] */
    public function getDisciplines($semesterID = null, $recordBookID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $recordBookID = $recordBookID ?: User::instance()->RecordBookID;
        
        $sql = 'CALL `Student_GetDisciplines`(:recordBookID, :semesterID)';
        $query = DB::query(Database::SELECT, $sql)
            ->param(':recordBookID', $recordBookID)
            ->param(':semesterID', $semesterID)
            ->execute();

        $list = [];
        foreach ($query as $data)
            $list[] = new Model_Discipline($data, true);

        return $list;
    }

    public function getTeachers($loadAll = false, $semesterID = null, $recordBookID = null) {
        // todo: don't load the full data at the Controller_Student_Index
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        $recordBookID = $recordBookID ?: User::instance()->RecordBookID;

        $sql = 'CALL `Student_GetTeachersList`(:recordBookID, :semesterID, :loadAll)';
        $result = DB::query(Database::SELECT, $sql)
            ->param(':recordBookID', $recordBookID)
            ->param(':semesterID', $semesterID)
            ->param(':loadAll', $loadAll)
            ->execute();

        $list = [];
        foreach ($result as $row) {
            $id = $row['DisciplineID'];
            $list[$id][] = $row;
        }
        return $list;
    }

    /**
     * @see        students_groups.IsStudyLeave
     * @deprecated cause it's not implemented yet
     * @return boolean true, if the student is on leave
     */
    public function inAcademicLeave() {
        throw new BadMethodCallException('Method is not implemented yet!');
    }

    /**
     * Send the student to an academic leave.
     * @return $this;
     */
    public function toAcademicLeave($semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;
        throw new InvalidArgumentException('Supply recordbook id instead of student id!');
        $sql = 'SELECT `ControlStudentGroup`(:id, -1, "leave", :semesterID)';


        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':semesterID', $semesterID)
            ->execute();

        return $this;
    }

    /**
     * Set the student's state and group in a semester.
     * @param $group int  group id
     * @return $this;
     */
    public function setState($group, $state="common", $semesterID = null) {
        $semesterID = $semesterID ?: User::instance()->SemesterID;;
        throw new InvalidArgumentException('Supply recordbook id instead of student id!');
        $sql = 'SELECT `ControlStudentGroup`(:id, :group, :state, :semesterID) AS Result';

        DB::query(Database::SELECT, $sql)
            ->param(':id', $this->ID)
            ->param(':group', (int) $group)
            ->param(':state', $state)
            ->param(':semesterID', $semesterID)
            ->execute();

        return $this;
    }

    public function transferIntoGroup($group, $semesterID = null) {
        return $this->setState($group, "common", $semesterID);
    }

}
